﻿using DevComponents.DotNetBar.Controls;
using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Runtime.InteropServices;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;

namespace centralController
{
    public partial class centralController : Form
    {
        /// <summary>
        /// 窗体初始化
        /// </summary>
        public centralController()
        {
            InitializeComponent();
            if (Monitor.Monitor.ins == null)
            {
                Monitor.Monitor.ins = new Monitor.Monitor(flowLayoutPanel1.Handle);
                Monitor.Monitor.ins.Start();
            }
            //进度条测试
            Task.Factory.StartNew(() =>
            {
                int temp = 0;
                while (true)
                {
                    if (temp != Monitor.Monitor.initializeState)
                    {
                        temp = Monitor.Monitor.initializeState;
                        progressBar1.Invoke(new Action(() => { progressBar1.Value = temp * 20; }));
                    }
                    if (temp == 5) { break; }
                    Thread.Sleep(200);
                }
            });
            //TimerCallback tc = new TimerCallback(RefreshRecords);
            //System.Threading.Timer t = new System.Threading.Timer(tc,null,5000,1000);
            System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
            timer.Interval = 10000;
            timer.Tick += new EventHandler(RefreshRecords);
            timer.Start();
            UpdateBasicInfo();
        }

        /// <summary>
        /// 结束按钮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_exit_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show("确定退出系统吗？", "温馨提示", MessageBoxButtons.OKCancel);
            if (result.Equals(DialogResult.OK))
            {
                Monitor.Monitor.ins.Stop();
                Close();
            }
        }
        /// <summary>
        /// 获取号牌测试
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            string ip = textBox1.Text;
            IPAddress temp;
            if (IPAddress.TryParse(ip, out temp))
            {
                string license = "";
                Task.Factory.StartNew(() =>
                {
                    license = Monitor.Monitor.numMachineLinker.GetLicensePlate(1);
                    if (textBox2.InvokeRequired)
                    {
                        textBox2.Invoke(new Action(() => { textBox2.Text = license; }));
                    }
                    else
                    {
                        textBox2.Text = license;
                    }

                });


            }
        }
        /// <summary>
        /// 手动连接PLC
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_linkPLC_Click(object sender, EventArgs e)
        {
            if (!Monitor.Monitor.PLC.isConnected)
            {
                Monitor.Monitor.PLC.PLCConnect();
            }
        }
        /// <summary>
        /// 刷新停车记录按钮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_refreshRecords_Click(object sender, EventArgs e)
        {
            RefreshRecords(null, null);
        }
        /// <summary>
        /// 刷新停车记录显示
        /// </summary>
        /// <param name="data"></param>
        private void RefreshRecords(object data, EventArgs e)
        {
            dataGridViewX1.Rows.Clear();
            List<object[]> list = Monitor.Monitor.GetParkingRecords();
            List<object[]>.Enumerator listEnumer = list.GetEnumerator();
            while (listEnumer.MoveNext())
            {
                dataGridViewX1.Rows.Add(listEnumer.Current);
            }
        }
        /// <summary>
        /// 导出excel文件
        /// </summary>
        /// <param name="dataGridView1"></param>
        /// <param name="path">绝对路径，"D:\\abc\\xxx.xls"</param>
        private void Export(DataGridViewX dataGridView1, string path)
        {
            try
            {
                if (dataGridView1.Rows.Count == 0)
                {
                    return;
                }
                Excel.Application excel = new Excel.Application();
                excel.Visible = false;
                Excel.Workbook workbook = excel.Workbooks.Add(true);
                Excel.Worksheet worksheet = (Excel.Worksheet)workbook.Worksheets[1];
                for (int i = 0; i < dataGridView1.Columns.Count; i++)
                {
                    if (dataGridView1.Columns[i].Visible == true)
                    {
                        worksheet.Cells[1, i + 1] = dataGridView1.Columns[i].HeaderText;
                    }
                }
                for (int i = 0; i < dataGridView1.Rows.Count - 1; i++)
                {
                    System.Windows.Forms.Application.DoEvents();
                    for (int j = 0; j < dataGridView1.Columns.Count; j++)
                    {
                        if (dataGridView1.Columns[j].Visible == true)
                        {
                            if (dataGridView1[j, i].ValueType == typeof(string))
                            {
                                worksheet.Cells[i + 2, j + 1] = "'" + dataGridView1[j, i].Value.ToString();
                            }
                            else
                            {
                                worksheet.Cells[i + 2, j + 1] = dataGridView1[j, i].Value.ToString();
                            }
                        }
                    }
                }
                //设置禁止弹出保存和覆盖的询问提示框  
                excel.DisplayAlerts = false;
                excel.AlertBeforeOverwriting = false;
                worksheet.Cells.Columns.AutoFit();
                //保存写入的数据，这里还没有保存到磁盘
                workbook.Saved = true;
                if (Directory.Exists(path))
                {

                    //设置新建文件路径及名称
                    string savePath = path + DateTime.Now.ToString("yyyy-MM-dd") + "停车记录.xls";

                    //创建文件
                    FileStream file = new FileStream(savePath, FileMode.CreateNew);

                    //关闭释放流，不然没办法写入数据
                    file.Close();
                    file.Dispose();

                    //保存到指定的路径
                    workbook.SaveCopyAs(savePath);
                }

                workbook.Close(false, Type.Missing, Type.Missing);

                //确保Excel进程关闭  
                excel.Quit();
                //释放 COM 对象
                Marshal.ReleaseComObject(worksheet);
                Marshal.ReleaseComObject(workbook);
                Marshal.ReleaseComObject(excel);
                excel = null;
                worksheet = null;
                workbook = null;
                GC.Collect();//如果不使用这条语,excel会无法正常退出
            }
            catch (Exception e) { Console.WriteLine("导出excel文件异常，" + e.Message); }

        }
        /// <summary>
        /// 更新基本信息
        /// </summary>
        private void UpdateBasicInfo()
        {
            string sysInfo = Monitor.Monitor.GetSysInfo();
            tbx_sysInfo.Text = sysInfo;
        }
        /// <summary>
        /// 导出停车记录按钮
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btn_exportRecords_Click(object sender, EventArgs e)
        {
            Export(dataGridViewX1, Directory.GetCurrentDirectory() + "\\");
            Console.WriteLine(Directory.GetCurrentDirectory() + "\\");
        }

        private void flowLayoutPanel1_Resize(object sender, EventArgs e)
        {
            Control.ControlCollection flpCC = flowLayoutPanel1.Controls;
            try
            {
                for (int i = 0; i < flpCC.Count; i++)
                {
                    flpCC[i].Width = (int)(flowLayoutPanel1.Width * 0.3);
                    flpCC[i].Height = (int)(flowLayoutPanel1.Height * 0.49);
                    Console.WriteLine("flp::" + flowLayoutPanel1.Width + "," + flowLayoutPanel1.Height);
                    //Console.WriteLine("tlp--"+flpCC[i].Width + "," + flpCC[i].Height);
                    Control.ControlCollection tlpCC = flpCC[i].Controls;
                    if (tlpCC.Count == 2)
                    {
                        tlpCC[0].Width = flpCC[i].Width - 5;
                        tlpCC[0].Height = (int)(flpCC[i].Height * 0.79);
                        tlpCC[1].Width = flpCC[i].Width - 5;
                        tlpCC[1].Height = (int)(flpCC[i].Height * 0.19);
                        //Console.WriteLine(tlpCC[0].Width + "," + tlpCC[0].Height);
                        //Console.WriteLine(tlpCC[1].Width + "," + tlpCC[1].Height);
                    }
                }
            }
            catch (Exception ex) { Console.WriteLine(ex.Message); }

        }
    }
}
